# Copyright (C) 2009, Mathieu PASQUET <kiorky@cryptelium.net>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the <ORGANIZATION> nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.




__docformat__ = 'restructuredtext en'

import os
import sys
import optparse

from minitage.core import core


usage = """
%(arg)s [Options] minibuildn  \t\t\t: Installs  package(s)
%(arg)s [Options] -j m  a b m ... n  \t\t: Installs  package(s) from 'm' to 'n'
%(arg)s [Options] -rm minibuild ... minibuildn  \t: Uninstall package(s)
%(arg)s [Options] -uU minibuild ... minibuildn  \t: Update/Upgrade package(s)
%(arg)s [Options] -RuU minibuild ... minibuildn  \t: Update/Upgrade/rebuild package(s)


IMPORTANT NODE: buildout is running by default in non newest mode, see -u option!
""" % {'arg': sys.argv[0]}


def do_read_options():
    """Parse the command line thought arguments
       and throws CliError if any error.
    Returns
        - `options` : the options to give to minimerge
            They are cli parsed but action [string] is added to the oject.
            action can be one of these :

                - install
                - delete
                - reinstall
        - `args` [list] : cli left args, in fact these are the packages to deal with.
    """

    default_action = 'install'
    path = sys.exec_prefix

    offline_help = 'Build offline, do not try to connect outside.'
    debug_help = 'Run in debug mode'
    jump_help = 'Squizze prior dependencies to the '\
                'minibuild specified in that option'
    fetchonly_help = 'Fetch the packages but do not build yet'
    fetchfirst_help = 'Fetch the packages first before building them'
    delete_help = 'Remove selected packages'
    reinstall_help = 'Unconditionnaly rebuild/reinstall packages'
    install_help = 'Installs packages (default action)'
    nodeps_help = 'Squizzes all dependencies'
    config_help = 'Alternate config file. By default it\'s searched in '\
                  '%s/etc/minimerge.cfg.' % sys.exec_prefix
    pretend_help = 'Do nothing, show what will be done'
    update_help = 'Update packages from where they come '\
            'prior to compilation step automaticly'
    ask_help = 'Do nothing, show what will be done and ask to continue'
    only_dependencies_help = 'Do actions onto dependencies, do not build the given packages'
    all_python_versions_help = 'Build python bindings for all python packages present in minitage'
    upgrade_help = 'Will try to rebuild already installed sofware. if '\
            'you need to be sure that all will be rebuilt (for buildout, it runs it in new mode whereas by default, buildout is running in non newest mode, activate '\
            'also the -R flag. If you want minimerge to update '\
            'the packages from where they come, please activate '\
            'also the -U flag.'

    actions = [
        optparse.make_option('-s', '--sync',
                             action='store_true', dest='sync',
                             help = nodeps_help),
        optparse.make_option('-i', '--install',
                             action='store_true', dest='install',
                             help = install_help),
        optparse.make_option('-U', '--update',
                             action='store_true', dest='update',
                             help = update_help),
        optparse.make_option('-u', '--upgrade',
                             action='store_true', dest='upgrade',
                             help = upgrade_help),
        optparse.make_option('-R', '--reinstall',
                             action='store_true', dest='reinstall',
                             help = reinstall_help),
        optparse.make_option('-E', '--generate-env',
                             action='store_true', dest='generate_env',
                             help =  '(re)generate instance .env shell file'), 
        optparse.make_option('--rm',
                             action='store_true', dest='delete',
                             help = delete_help),
    ]
    modifiers = [
        optparse.make_option('-N', '--nodeps',
                             action='store_true', dest='nodeps',
                             help = nodeps_help),
        optparse.make_option('-j', '--jump',
                             action='store', dest='jump',
                             help = jump_help),
        optparse.make_option('-F', '--fetchonly',
                             action='store_true', dest='fetchonly',
                             help = fetchonly_help),
        optparse.make_option('-f', '--fetchfirst',
                             action='store_true', dest='fetchfirst',
                             help = fetchfirst_help),
        optparse.make_option('-k', '--use-binaries',
                             action='store_true', dest='binary',
                             help = 'Search and install binaries instead of classicly compile.'),
        optparse.make_option('-o', '--offline',
                             action='store_true', dest='offline',
                             help = offline_help),
    ]
    flags = [
        optparse.make_option('-p', '--pretend',
                             action='store_true', dest='pretend',
                             help = pretend_help),
        optparse.make_option('--only-dependencies',
                             action='store_true', dest='only_dependencies',
                             help = only_dependencies_help), 
        optparse.make_option('--all-python-versionss',
                             action='store_true', dest='all_python_versions',
                             help = all_python_versions_help),  
        optparse.make_option('-a', '--ask',
                             action='store_true', dest='ask',
                             help = ask_help),
        optparse.make_option('--skip-self-upgrade',
                             action='store_true', dest='skip_self_upgrade',
                             help = 'Do not do minitage self upgrades.'),
        optparse.make_option('--reinstall-minilays',
                             action='store_true', dest='reinstall_minilays',
                             help = 'Re download minilays.'),
        optparse.make_option('-v', '--verbose',
                             action='store_true', dest='verbose',
                             help = 'Be verbose.'),
        optparse.make_option('-c', '--config',
                             action='store', dest='config',
                             help = config_help),
        optparse.make_option('-d', '--debug',
                             action='store_true', dest='debug',
                             help = debug_help),
    ]
    parser = optparse.OptionParser(version=core.__version__, usage=usage)
    flags_group = optparse.OptionGroup(parser, 'Flags')
    modifiers_group = optparse.OptionGroup(parser, 'Modifiers')
    actions_group = optparse.OptionGroup(parser, 'Actions')
    [[group.add_option(o) for o in opts]
     for group, opts in [(actions_group, actions),
                         (modifiers_group, modifiers),
                         (flags_group, flags)]
    ]
    [parser.add_option_group(group) for group in [actions_group, modifiers_group, flags_group]]
    (options, args) = parser.parse_args()

    if (options.reinstall and options.delete) or\
       (options.fetchonly and options.offline) or \
       (options.jump and options.nodeps):
        raise core.ConflictModesError('You are using conflicting modes')

    if (not args and len(sys.argv) > 1) and not (options.sync or options.reinstall_minilays):
        message = 'You must precise which packages you want to deal with'
        raise core.NoPackagesError(message)

    if len(sys.argv) == 1:
        print 'minimerge v%s' % parser.version
        parser.print_usage()
        print '\'%s --help\' for more inforamtion on usage.' % sys.argv[0]

    actionsCount = 0
    for action in [options.reinstall, options.install, options.delete]:
        if action:
            actionsCount += 1
    if actionsCount > 1:
        message = 'You must precise only one action at a time'
        raise core.TooMuchActionsError(message)

    if options.delete:
        options.action = 'delete'
    elif options.reinstall:
        options.action = 'reinstall'
    elif options.sync:
        options.action = 'sync'
    elif options.install:
        options.action = 'install'
    elif options.generate_env:
        options.action = 'generate_env' 
    else:
        options.action = default_action

    if not options.config:
        for file in ['%s/etc/minimerge.cfg' % path, '~/.minimerge.cfg']:
            file = os.path.expanduser(file)
            if os.path.isfile(file):
                options.config = file
                break

    # be sure to be with full path object.
    if not options.config:
        message = 'The configuration file specified does not exist'
        raise core.InvalidConfigFileError(message)
    if not os.path.isfile(options.config):
        message = 'The configuration file specified does not exist'
        raise core.InvalidConfigFileError(message)
    options.config = os.path.expanduser(options.config)

    minimerge_options = {
        'action': options.action,
        'ask': options.ask,
        'config': options.config,
        'debug': options.debug,
        'fetchfirst': options.fetchfirst,
        'fetchonly': options.fetchonly,
        'jump': options.jump,
        'nodeps': options.nodeps,
        'offline': options.offline,
        'packages': args,
        'path': path,
        'pretend': options.pretend,
        'update': options.update,
        'upgrade': options.upgrade,
        'verbose': options.verbose,
        'only_dependencies': options.only_dependencies,
        'all_python_versions': options.all_python_versions,
        'reinstall_minilays': options.reinstall_minilays,
        'binary': options.binary,
        'skip_self_upgrade': options.skip_self_upgrade,
    }
    return minimerge_options




# Copyright (C) 2009, Mathieu PASQUET <kiorky@cryptelium.net>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the <ORGANIZATION> nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

__docformat__ = 'restructuredtext en'

import os
import sys
import unittest
import tarfile
from minitage.core.common import first_run

eggs = os.environ.get('MINITAGE_CORE_EGG_PATH', None)
setup = os.environ.get('MINITAGE_CORE_SETUPPY', None)
#if not setup:
#    raise Exception("Please set the 'MINITAGE_CORE_SETUPPY' variable pointing to the setup.py file of the minitage distribution")

def createMinitageEnv(directory):
    """Initialise a minitage in a particular directory."""

    if os.path.exists(os.path.expanduser(directory)):
        raise Exception("Please (re)move %s before test" % directory)
    # faking dev mode
    os.chdir('/')
    module =  os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..', '..'))
    mc =  os.path.dirname(module)
    os.system("""
              mkdir %(path)s
              cd /
              virtualenv %(path)s --no-site-packages
              source %(path)s/bin/activate
              easy_install virtualenv
              # can be python-ver or python
              $(ls %(path)s/bin/easy_install) -Uf "%(eggs)s" zc.buildout setuptools iniparse
              pushd %(mc)s
              python=$(ls -1 %(path)s/bin/python*|head -n1)
              $python setup.py develop
              $python -c 'from minitage.core.common import first_run;first_run()'
              """ % {
                  'eggs': eggs,
                  'path': directory,
                  'setup': setup,
                  'module': module,
                  'mc': mc,
              }
             )

def write(file, s):
    """Write content to a file."""

    f = open(file,'w')
    f.write(s)
    f.flush()
    f.close()


def bootstrap_buildout(dir):
    """Initialise the bin/buildout file."""

    template = """
##############################################################################
#
# Copyright (c) 2006 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

import os, shutil, sys, tempfile, urllib2
from optparse import OptionParser

tmpeggs = tempfile.mkdtemp()

is_jython = sys.platform.startswith('java')

# parsing arguments
parser = OptionParser()
parser.add_option("-v", "--version", dest="version",
                          help="use a specific zc.buildout version")
parser.add_option("-d", "--distribute",
                   action="store_true", dest="distribute", default=False,
                   help="Use Disribute rather than Setuptools.")

parser.add_option("-c", None, action="store", dest="config_file",
                   help=("Specify the path to the buildout configuration "
                         "file to be used."))

options, args = parser.parse_args()

# if -c was provided, we push it back into args for buildout' main function
if options.config_file is not None:
    args += ['-c', options.config_file]

if options.version is not None:
    VERSION = '==%s' % options.version
else:
    VERSION = ''

USE_DISTRIBUTE = options.distribute
args = args + ['bootstrap']

to_reload = False
try:
    import pkg_resources
    if not hasattr(pkg_resources, '_distribute'):
        to_reload = True
        raise ImportError
except ImportError:
    ez = {}
    if USE_DISTRIBUTE:
        exec urllib2.urlopen('http://python-distribute.org/distribute_setup.py'
                         ).read() in ez
        ez['use_setuptools'](to_dir=tmpeggs, download_delay=0, no_fake=True)
    else:
        exec urllib2.urlopen('http://peak.telecommunity.com/dist/ez_setup.py'
                             ).read() in ez
        ez['use_setuptools'](to_dir=tmpeggs, download_delay=0)

    if to_reload:
        reload(pkg_resources)
    else:
        import pkg_resources

if sys.platform == 'win32':
    def quote(c):
        if ' ' in c:
            return '"%s"' % c # work around spawn lamosity on windows
        else:
            return c
else:
    def quote (c):
        return c

cmd = 'from setuptools.command.easy_install import main; main()'
ws  = pkg_resources.working_set

if USE_DISTRIBUTE:
    requirement = 'distribute'
else:
    requirement = 'setuptools'

if is_jython:
    import subprocess

    assert subprocess.Popen([sys.executable] + ['-c', quote(cmd), '-mqNxd',
           quote(tmpeggs), 'zc.buildout' + VERSION],
           env=dict(os.environ,
               PYTHONPATH=
               ws.find(pkg_resources.Requirement.parse(requirement)).location
               ),
           ).wait() == 0

else:
    assert os.spawnle(
        os.P_WAIT, sys.executable, quote (sys.executable),
        '-c', quote (cmd), '-mqNxd', quote (tmpeggs), 'zc.buildout' + VERSION,
        dict(os.environ,
            PYTHONPATH=
            ws.find(pkg_resources.Requirement.parse(requirement)).location
            ),
        ) == 0

ws.add_entry(tmpeggs)
ws.require('zc.buildout' + VERSION)
import zc.buildout.buildout
zc.buildout.buildout.main(args)
shutil.rmtree(tmpeggs) 
"""

    cwd = os.getcwd()
    os.chdir(dir)
    write('bootstrap.py', template)
    os.system('%s bootstrap.py' % sys.executable)
    os.chdir(cwd)


def make_dummy_buildoutdir(ipath):
    os.makedirs(ipath)
    os.chdir(ipath)
    write('buildout.cfg', """
[makers]
[buildout]
options = -c buildout.cfg  -vvvvvv
parts = x
        z
develop = .
[part]
recipe = toto:part
[site-packages-2.4]
recipe = toto:py24
[site-packages-2.5]
recipe = toto:py25
[z]
recipe = toto:luu
[y]
recipe = toto:bar
[x]
recipe = toto """)
    write('setup.py', """
from setuptools import setup
setup(
          name='toto',
          entry_points= {
          'zc.buildout': [
              'default = toto:test',
              'luu = tutu:test',
              'bar = tata:test',
              'py25 = py25:test',
              'py24 = py24:test',
              'part = part:test',
             ]
         }
) """)

    write('toto.py', """
class test:
    def __init__(self,a, b, c):
        pass

    def install(a):
        print "foo" """)
    write('tata.py', """
class test:
    def __init__(self,a, b, c):
        pass

    def install(a):
        open('testbar','w').write('foo') """)
    write('tutu.py', """
class test:
    def __init__(self,a, b, c):
        pass

    def install(a):
        open('testres','w').write('bar') """)

    write('py25.py', """
class test:
    def __init__(self,a, b, c):
        pass

    def install(a):
        open('testres2.5','w').write('2.5') """)

    write('py24.py', """
class test:
    def __init__(self,a, b, c):
        pass

    def install(a):
        open('testres2.4','w').write('2.4') """)
    write('part.py', """
class test:
    def __init__(self,a, b, c):
        pass

    def install(a):
        open('testres','w').write('part') """)
    bootstrap_buildout(ipath)

def createPackage(supath):
    dest = os.path.join(supath, 'buildout-package')
    tarp = os.path.join(supath, 'buildout-package.tgz')
    make_dummy_buildoutdir(dest)
    tar = tarfile.open(tarp, "w:gz")
    tar.add(dest, '.')
    tar.close()
    return tarp

def test_suite():            
    suite = unittest.TestSuite()
    return suite  





# -*- coding: utf-8 -*-
# Copyright (C) 2009, Mathieu PASQUET <kiorky@cryptelium.net>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the <ORGANIZATION> nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.



__docformat__ = 'restructuredtext en'

import unittest
import os
import tempfile
from cStringIO import StringIO

from minitage.core.core import newline


def write(file, content):
    fic = open(file, 'w')
    fic.write(content)
    fic.flush()
    fic.close()

class testNewLine(unittest.TestCase):
    """testNewLine"""

    def setUp(self):
        """."""
        self.file = tempfile.mkstemp()[1]

    def tearDown(self):
        """."""
        if os.path.exists(self.file):
            os.unlink(self.file)

    def testNewLine(self):
        """testNewLine"""
        write(self.file,
"""
tata
test
"""
        )
        newline(self.file)
        self.assertEquals(
            '\ntata\ntest\n\n',
            open(self.file).read()
        )
        write(self.file,
"""
tata
test"""
        )
        newline(self.file)
        self.assertEquals(
            '\ntata\ntest\n\n',
            open(self.file).read()
        )


    def testEmptyFile(self):
        """testEmptyFile"""
        write(self.file,
              """"""
        )
        newline(self.file)
        self.assertEquals(
            '\n',
            open(self.file).read()
        )

    def testUtf8(self):
        """tUtf8"""
        write(self.file,
              """çà@~#ø€ç±ø"""
        )
        newline(self.file)
        self.assertEquals(
            '\xc3\xa7\xc3\xa0@~#\xc3\xb8\xe2\x82\xac\xc3\xa7\xc2\xb1\xc3\xb8\n\n',
            open(self.file).read()
        )

    def multipleLines(self):
        """multipleLines"""
        write(self.file,
"""
tata
test




"""
        )
        newline(self.file)
        self.assertEquals(
            '\ntata\ntest\n\n',
            open(self.file).read()
        )



def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(testNewLine))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(test_suite())






"""
Generic Test case
"""
__docformat__ = 'restructuredtext'

import unittest
import doctest
import sys
import os
import tempfile
import shutil
import subprocess
from os import makedirs as mkdir
from shutil import copy
import logging
from distutils.dir_util import copy_tree


logging.basicConfig()
import minitage.core.tests
current_dir =  minitage.core.tests.__path__[0]

def rmdir(*args):
    dirname = os.path.join(*args)
    if os.path.isdir(dirname):
        shutil.rmtree(dirname)

def sh(cmd, in_data=None):
    _cmd = cmd
    print cmd
    p = subprocess.Popen([_cmd], shell=True,
                         stdin=subprocess.PIPE,
                         stdout=subprocess.PIPE, close_fds=True)

    if in_data is not None:
        p.stdin.write(in_data)

    p.stdin.close()

    print p.stdout.read()

def ls(*args):
    dirname = os.path.join(*args)
    if os.path.isdir(dirname):
        filenames = os.listdir(dirname)
        for filename in sorted(filenames):
            print filename
    else:
        print 'No directory named %s' % dirname

def cd(*args):
    dirname = os.path.join(*args)
    os.chdir(dirname)

def config(filename):
    return os.path.join(current_dir, filename)


def cat(*args, **kwargs):
    filename = os.path.join(*args)
    if os.path.isfile(filename):
        data = open(filename).read()
        if kwargs.get('returndata', False):
           return data
        print data
    else:
        print 'No file named %s' % filename

def touch(*args, **kwargs):
    filename = os.path.join(*args)
    open(filename, 'w').write(kwargs.get('data',''))


#execdir = os.path.abspath(os.path.dirname(sys.executable))
tempdir = os.getenv('TEMP','/tmp')

class DoctestLayer(object):
    """"""

def default_setUp(test):
    globs = test.globs
    for p in [globs['p'],
              globs['p2'],
              globs['p3'],
              globs['wc'],]:
        if os.path.exists(p):
            shutil.rmtree(p)
        os.makedirs(p)

def default_tearDown(test):
    globs = test.globs
    if os.path.exists(globs['p']):
        shutil.rmtree(globs['p3'])
        shutil.rmtree(globs['p2'])
        shutil.rmtree(globs['wc'])

def doc_suite(test_dir, setUp=None, tearDown=None, globs=None):
    """Returns a test suite, based on doctests found in /doctest."""
    suite = []
    tmpdir = tempfile.mkdtemp()
    if globs is None:
        globs = globals()
        globs['p'] = tmpdir
        globs['path2'] = globs['p2'] = os.path.join(tmpdir, 'p2')
        globs['path3'] = globs['p3'] = os.path.join(tmpdir, 'p3')
        globs['wc'] = os.path.join(tmpdir, 'wc')

    flags = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE |
             doctest.REPORT_ONLY_FIRST_FAILURE)

    doctest_dir = test_dir

    # filtering files on extension
    docs = [os.path.join(doctest_dir, doc) for doc in
            os.listdir(doctest_dir) if doc.endswith('.txt')
           # if [selectable
           #     for selectable in ['bzr','hg', 'git']
           #     if selectable in doc]
           ]

    if not setUp:
        setUp = default_setUp

    if not tearDown:
        tearDown = default_tearDown

    for ftest in docs:
        test = doctest.DocFileSuite(
            ftest, optionflags=flags,
            globs=globs, setUp=setUp,
            tearDown=tearDown,
            module_relative=False
        )
        test.layer = DoctestLayer
        suite.append(test)

    return unittest.TestSuite(suite)

def test_suite():
    """returns the test suite"""
    return doc_suite(current_dir)

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(test_suite()) 




#!/usr/bin/env python

# Copyright (C) 2009, Mathieu PASQUET <mpa@makina-corpus.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

__docformat__ = 'restructuredtext en'

import os
import sys

import subprocess

UPDATES = {}

def upperl(minitage):
    minitage.logger.info('Creating CPAN')
    for dir in (os.path.join(minitage._prefix, 'cpan', '5.8'),):
        if not os.path.exists(dir):
            os.makedirs(dir)
    # reinstall perl if any
    if os.path.exists(
        os.path.join(minitage._prefix,
                     'dependencies',
                     'perl-5.8'
                    )
    ):
        minitage.reinstall_packages(['perl-5.8'])

def reinstall_minilays(self):
    self.logger.info('Reinstallating minitage default minileys !')
    self.reinstall_minilays()

def updateHistory(self, force=False):
        """Copy the current minibuild prior to minimerge -s
        if minitage installation does not have any history yet"""
        hd = os.path.join(
            self._prefix, '.minitage_history'
        )
        if not os.path.exists(hd):
            force = True
        if force and not self.first_run:
            self.logger.info('Migrating existing and '
                             'installed packages to use the '
                             'minitage history system.')
            for minilay in self._minilays:
                minilay.load()
                for mb in minilay:
                    minibuild = minilay[mb]
                    if not minibuild.name.startswith('.'):
                        ip = self.get_install_path(minibuild)
                        if os.path.exists(ip):
                            if (not self.is_installed(minibuild)
                                and (len(os.listdir(ip))>0)):
                                self.set_package_mark(minibuild,
                                                      'install',
                                                      'install')
                                self.record_minibuild(minibuild)
            fic = open(hd, 'w')
            fic.write("")
            fic.close()
            self.logger.info('Migration complete, please restart minimerge.')
            self._action = None
            self._packages = []

def upgrademinilaysurl(self):
    pass

def reinstall_core_libs(self):
    self.reinstall_packages(['readline-6', 'ncurses-5', 'openssl-1'])

def migrate_minibuilds_to_new_libs_2019(self):
    migrate_minibuilds_to_new_libs(self,
        {'openssl-0.9': 'openssl-1',
         'ncurses-5.6': 'ncurses-5',
         'readline-5.2': 'readline-6',
         'postgis-1.4': 'postgis-1.5',
        })

def migrate_minibuilds_to_new_libs_2018(self):
    migrate_minibuilds_to_new_libs(self,
        {'libxml2-2.6': 'libxml2-2.7',
         'py-libxml2-2.6': 'py-libxml2-2.7',
        })

def migrate_minibuilds_to_new_libs(self, deps_ups):
    updated = {}
    minilays = [mn
                for mn in self._minilays
                #if not mn in ['dependencies', 'eggs']
               ]
    for minilay in minilays:
        mn = os.path.basename(minilay.path)
        minilay.load()
        for mbk in minilay:
            mb = minilay[mbk]
            mb.load()
            for old in deps_ups:
                new = deps_ups[old]
                dependencies = mb.raw_dependencies
                if old in dependencies:
                    i = dependencies.index(old)
                    dependencies.pop(i)
                    dependencies.insert(i, new)
                    mb.revision += 1
                    mb.write(dependencies=dependencies,
                            revision=mb.revision)
                    if not mb.path in updated:
                        updated[mb.path] = mb
    # try to regenerate .env
    for mpath in updated:
        mb = updated[mpath]
        path = self.get_install_path(mb)
        env = os.path.join(path, 'sys', 'share', 'minitage', 'minitage.env')
        if os.path.exists(env):
            self.generate_env(mb)

#UPDATES['1.0.11'] = [#upperl,
#                     reinstall_minilays]

UPDATES['2.0'] = [updateHistory,
                  reinstall_minilays,
                  ]
UPDATES['2.0.18'] = [
    reinstall_minilays,
    migrate_minibuilds_to_new_libs_2018 ,
]
UPDATES['2.0.19'] = [
    migrate_minibuilds_to_new_libs_2019 ,
]

# vim:set et sts=4 ts=4 tw=80:



# Copyright (C) 2009, Mathieu PASQUET <kiorky@cryptelium.net>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the <ORGANIZATION> nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.



__docformat__ = 'restructuredtext en'
import unittest
from minitage.core.tests.makers import (test_interfaces,
                    test_buildout,
                   )
def test_suite():
    suite = unittest.TestSuite()
    for m in (test_interfaces,
              test_buildout,
             ):
        suite.addTest(m.test_suite())
    return suite
if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(test_suite()) 

# vim:set et sts=4 ts=4 tw=80:



# Copyright (C) 2009, Mathieu PASQUET <kiorky@cryptelium.net>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the <ORGANIZATION> nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

__docformat__ = 'restructuredtext en'

import os
import subprocess
import re
import datetime
import logging
from distutils.dir_util import copy_tree

from minitage.core.fetchers import interfaces

__logger__ = 'minitage.fetchers.scm'

class OfflineModeRestrictionError(interfaces.IFetcherError):
    """Restriction error in offline mode."""

class HgFetcher(interfaces.IFetcher):
    """ Mercurial Fetcher.
    Example::
        >>> import minitage.core.fetchers.scm
        >>> hg = scm.HgFetcher()
        >>> hg.fetch_or_update('http://uri','/dir',{revision='tip'})
    """

    def __init__(self, config = None):
        self.config =  config
        interfaces.IFetcher.__init__(self, 'Mercurial', 'hg', config, '.hg', 'tip')
        self.log = logging.getLogger(__logger__)

    def checkout(self, dest, uri, opts, verbose=True):
        args = opts.get('args', '')
        if not verbose:
            args += ' -q '
        self._scm_cmd('clone %s %s %s' % (args, uri, dest), verbose)

    def update_wc(self, dest, uri, opts, verbose=True):
        if not uri:
            uri = ''
        args = opts.get('args', '')
        vargs = ''
        if not verbose:
            vargs += ' -q '
        self._scm_cmd('pull %s -f %s %s -R %s' % (vargs, uri, args, dest), verbose)

    def goto_revision(self, dest, uri, opts, verbose=True):
        args = opts.get('goto-revision-args', '')
        if not verbose:
            args += ' -q '
        if 'revision' in opts:
            args += '-C -r%s' % opts['revision']
        self._scm_cmd('up %s -R %s' % (args, dest), verbose)

    def is_valid_src_uri(self, uri):
        """See interface."""
        match = interfaces.URI_REGEX.match(uri)
        if match \
           and (match.groups()[2] in ['file', 'hg', 'ssh', 'http', 'https', '/']
                or match.groups()[0] == '/'):
            return True
        return False

    def match(self, switch):
        """See interface."""
        if switch == 'hg':
            return True
        return False

    def get_uri(self, dest):
        """get Mercurial url"""
        self._check_scm_presence()
        try:
            cwd = os.getcwd()
            os.chdir(dest)
            self.log.debug('Running %s %s in %s' % (
                self.executable,
                'showconfig |grep paths.default',
                dest
            ))
            process = subprocess.Popen(
                '%s %s' % (
                    self.executable,
                    'showconfig |grep paths.default'
                ),
                shell = True, stdout=subprocess.PIPE
            )
            ret = process.wait()
            if ret != 0:
                message = '%s failed to achieve correctly.' % self.name
                raise interfaces.FetcherRuntimeError(message)
            dest_uri = re.sub('([^=]*=)\s*(.*)',
                          '\\2',
                          process.stdout.read().strip()
                         )
            os.chdir(cwd)
            return dest_uri
        except Exception, instance:
            os.chdir(cwd)
            raise instance

    def _has_uri_changed(self, dest, uri):
        """See interface."""
        # file is removed on the local uris
        uri = uri.replace('file://', '')
        # in case we were not hg before
        if not os.path.isdir(os.path.join(dest, self.metadata_directory)):
            return True
        if self.warn_trailing_slash(dest, uri):
            return False
        if uri != self.get_uri(dest):
            return True
        return False


class SvnFetcher(interfaces.IFetcher):
    """Subversion Fetcher.
    Example::
        >>> import minitage.core.fetchers.scm
        >>> svn = scm.SvnFetcher()
        >>> svn.fetch_or_update('http://uri','/dir',{revision='HEAD'})
    """

    def __init__(self, config = None):
        self.config =  config
        interfaces.IFetcher.__init__(self, 'subversion', 'svn', config, '.svn',
                                    'HEAD')
        self.log = logging.getLogger(__logger__)

    def checkout(self, dest, uri, opts, verbose=True):
        args = opts.get('args', '')
        if not verbose:
            args += ' -q '
        args = '%s %s' % (args, opts.get('goto-revision-args', ''))
        if 'revision' in opts:
            args += '-r %s' % opts['revision']
        self._scm_cmd('co %s %s %s' % (args, uri, dest), verbose=True)

    def update_wc(self, dest, uri, opts, verbose=True):
        args = opts.get('args', '')
        if not verbose:
            args += ' -q '
        args = '%s %s' % (args, opts.get('goto-revision-args', ''))
        if 'revision' in opts:
            args += '-r %s' % opts['revision']
        self._scm_cmd('up %s %s' % (args, dest), verbose)

    def goto_revision(self, dest, uri, opts, verbose,  passive = True):
        if not passive:
            self.update_wc(dest, uri, opts, verbose)

    def is_valid_src_uri(self, uri):
        """See interface."""
        match = interfaces.URI_REGEX.match(uri)
        if match \
           and match.groups()[2] \
           in ['file', 'svn', 'svn+ssh', 'http', 'https']:
            return True
        return False

    def match(self, switch):
        """See interface."""
        if switch == 'svn':
            return True
        return False

    def get_uri(self, dest):
        """Get url."""
        self._check_scm_presence()
        process = subprocess.Popen(
            '%s %s' % (
                self.executable,
                'info %s|grep -i url' % dest
            ),
            shell = True, stdout=subprocess.PIPE
        )
        ret = process.wait()
        # we werent svn
        if ret != 0:
            return None
        return re.sub(
            '([^:]*:)\s*(.*)', '\\2',
            process.stdout.read().strip()
        )

    def _has_uri_changed(self, dest, uri):
        """See interface."""
        if self.warn_trailing_slash(dest, uri):
            return False
        if uri != self.get_uri(dest):
            return True
        return False

class BzrFetcher(interfaces.IFetcher):
    """ Bazaar Fetcher.
    Example::
        >>> import minitage.core.fetchers.scm
        >>> bzr = scm.BzrFetcher()
        >>> bzr.fetch_or_update('http://uri','/dir',{revision='last:1'})
    """

    def __init__(self, config = None):
        self.config =  config
        interfaces.IFetcher.__init__(self, 'bazaar', 'bzr', config, '.bzr',
                                     'last:1')
        self.log = logging.getLogger(__logger__)

    def checkout(self, dest, uri, opts, verbose=True):
        args = opts.get('args', '')
        if not verbose:
            args += ' -q '
        self._scm_cmd('checkout %s %s %s' % (args, uri, dest), verbose)

    def update_wc(self, dest, uri, opts, verbose=True):
        args = opts.get('args', '')
        if not uri:
            uri = ''
        if not verbose:
            args += ' -q '
        self._scm_cmd('pull %s %s -d %s' % (args, uri, dest), verbose)

    def goto_revision(self, dest, uri, opts, verbose=True):
        args = opts.get('goto-revision-args', '')
        if not verbose:
            args += ' -q '
        if 'revision' in opts:
            args += '--overwrite -r%s' % opts['revision']
            self._scm_cmd('pull %s %s -d %s' % (
                args, uri, dest), verbose
            )

    def is_valid_src_uri(self, uri):
        """See interface."""
        match = interfaces.URI_REGEX.match(uri)
        bzrmatch = re.compile('[a-zA-Z1-9]*:(.*)').match(uri)
        if match \
           and match.groups()[2] \
           in ['file', 'bzr', 'sftp', 'http',
               'https', 'bzr+http', 'bzr+https',
               'bzr+ssh', 'svn+file',
               'svn', 'svn+http', 'svn+https'] or bzrmatch:
            return True
        return False

    def match(self, switch):
        """See interface."""
        if switch == 'bzr':
            return True
        return False

    def get_uri(self, dest):
        """get bazaar url"""
        self._check_scm_presence()
        try:
            cwd = os.getcwd()
            os.chdir(dest)
            self.log.debug('Running %s %s in %s' % (
                self.executable,
                ' info 2>&1|egrep "(checkout of branch|parent branch)"|cut -d:  -f 2,3',
                dest
            ))
            process = subprocess.Popen(
                '%s %s' % (
                    self.executable,
                    ' info 2>&1|egrep "(checkout of branch|parent branch)"|cut -d:  -f 2,3',
                ),
                shell = True, stdout=subprocess.PIPE
            )
            ret = process.wait()
            if ret != 0:
                message = '%s failed to achieve correctly.' % self.name
                raise interfaces.FetcherRuntimeError(message)
            dest_uri = re.sub(
                '([^=]*=)\s*(.*)',
                '\\2',
                process.stdout.read().strip()
            )
            if '\n' in dest_uri:
                # return 'checkout branch'
                dest_uri = dest_uri.split('\n')[0]
            os.chdir(cwd)
            return dest_uri
        except Exception, instance:
            os.chdir(cwd)
            raise instance

    def _has_uri_changed(self, dest, uri):
        """See interface."""
        # file is removed on the local uris
        uri = uri.replace('file://', '')
        repouri = self.get_uri(dest).replace('file://', '')
        # in case we were not bzr before
        if not os.path.isdir(os.path.join(dest, self.metadata_directory)):
            return True
        if self.warn_trailing_slash(dest, uri):
            return False
        if uri != repouri:
            return True
        return False

class GitFetcher(interfaces.IFetcher):
    """ Bazaar Fetcher.
    Example::
        >>> import minitage.core.fetchers.scm
        >>> git = scm.GitFetcher()
        >>> git.fetch_or_update('http://uri','/dir',{revision='HEAD'})
    """

    def __init__(self, config = None):
        self.config =  config
        interfaces.IFetcher.__init__(self, 'git', 'git', config, '.git', 'HEAD')
        self.log = logging.getLogger(__logger__)

    def checkout(self, dest, uri, opts, verbose=True):
        args = opts.get('args', '')
        if not verbose:
            args += ' -q '
        self._scm_cmd('clone  %s %s %s' % (args, uri, dest), verbose)

    def update_wc(self, dest, uri, opts, verbose=True):
        args = opts.get('args', '')
        if not verbose:
            args += ' -q '
        cwd = os.getcwd()
        os.chdir(dest)
        if not uri or (not self._has_uri_changed(dest, uri)):
            uri = ''
        try:
            self._scm_cmd('pull %s %s' % (args, uri), verbose)
        except Exception, e:
            os.chdir(cwd)
        os.chdir(cwd)

    def goto_revision(self, dest, uri, opts, verbose=True):
        args = opts.get('goto-revision-args', '')
        if not verbose:
            args += ' -q '
        if 'revision' in opts:
            cwd = os.getcwd()
            os.chdir(dest)
            self._scm_cmd(
                'reset %s --hard %s' % (
                    args,
                    opts['revision'],
                ),
                verbose
            )
            os.chdir(cwd)

    def is_valid_src_uri(self, uri):
        """See interface."""
        match = interfaces.URI_REGEX.match(uri)
        gitmatch = re.compile('[a-zA-Z1-9]*:(.*)').match(uri)
        if match \
           and match.groups()[2] \
           in ['file', 'git', 'rsync', 'http',
               'https', 'svn'] or gitmatch:
            return True
        return False

    def match(self, switch):
        """See interface."""
        if switch == 'git':
            return True
        return False


    def get_uri(self, dest):
        """get git url"""
        self._check_scm_presence()
        try:
            cwd = os.getcwd()
            os.chdir(dest)
            self.log.debug('Running %s %s in %s' % (
                self.executable,
                'config --get remote.origin.url',
                dest
            ))
            process = subprocess.Popen('%s config --get remote.origin.url' % self.executable,
                                       shell = True, stdout=subprocess.PIPE)
            ret = process.wait()
            if ret >1 :
                message = '%s failed to achieve correctly.' % self.name
                raise interfaces.FetcherRuntimeError(message)
            dest_uri = process.stdout.read().strip()
            os.chdir(cwd)
            return dest_uri
        except Exception, instance:
            os.chdir(cwd)
            raise instance

    def _has_uri_changed(self, dest, uri):
        """See interface."""
        # in case we were not git before
        if not os.path.isdir(os.path.join(dest, self.metadata_directory)):
            return True
        if self.warn_trailing_slash(dest, uri):
            return False
        if uri != self.get_uri(dest):
            return True
        return False

# vim:set et sts=4 ts=4 tw=80:



# Copyright (C) 2009, Mathieu PASQUET <kiorky@cryptelium.net>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the <ORGANIZATION> nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.



__docformat__ = 'restructuredtext en'

import unittest
import os
import sys
import shutil
import optparse
import ConfigParser
import tempfile

from minitage.core import api, cli, objects, core
from minitage.core import update
from minitage.core.tests import test_common
from minitage.core import tests
from minitage.core import version


__cwd__ = os.getcwd()


path = '%s/test' % os.path.expanduser(tempfile.mkdtemp())
testopts = dict(path=path)
minilay = '%(path)s/minilays/myminilay1' % testopts
class testMinimerge(unittest.TestCase):
    """testMinimerge"""

    def setUp(self):
        """."""
        os.chdir(__cwd__)
        self.minimerge10 = tempfile.mkstemp()[1]
        thisp = os.path.dirname('__file__')
        shutil.copy2(
            os.path.join(tests.__path__[0], 'minimerge_10.cfg'),
            self.minimerge10
        )
        test_common.createMinitageEnv(path)
        os.makedirs(minilay)

        minibuilds = [
"""
[minibuild]
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=eggs
""",
"""
[minibuild]
dependencies=minibuild-0
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=eggs
""",
"""
[minibuild]
dependencies=minibuild-4 minibuild-1
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=eggs
""",
"""
[minibuild]
dependencies=minibuild-2
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=eggs
""",
"""
[minibuild]
dependencies=minibuild-0
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=eggs
""",
"""
[minibuild]
dependencies=minibuild-7
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=eggs
""",
"""
[minibuild]
dependencies=minibuild-5
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=eggs
""",
"""
[minibuild]
dependencies=minibuild-6
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=eggs
""",
"""
[minibuild]
dependencies=minibuild-8
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=eggs
""",
"""
[minibuild]
dependencies=minibuild-0 minibuild-3
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=eggs
""",
"""
[minibuild]
dependencies=minibuild-11
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=eggs
""",
"""
[minibuild]
dependencies=minibuild-12
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=eggs
""",
"""
[minibuild]
dependencies=minibuild-13
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=eggs
""",
"""
[minibuild]
dependencies=minibuild-10
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=eggs
""" ]


        pythonmbs = [
#1000
"""
[minibuild]
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=dependencies
""", #1001
"""
[minibuild]
dependencies=meta-python minibuild-1005
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=dependencies
""", #1002
"""
[minibuild]
dependencies=python-2.4 minibuild-1005
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=dependencies
""", #1003
"""
[minibuild]
dependencies=python-2.5 minibuild-1005
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=dependencies
""", #1004
"""
[minibuild]
dependencies=minibuild-1005
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=dependencies
""", #1005
"""
[minibuild]
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=eggs
""",
]


        for index, minibuild in enumerate(minibuilds):
            open('%s/minibuild-%s' % (minilay, index), 'w').write(minibuild)

        os.system("""cd %s ;
                  hg init;
                  hg add ;
                  hg ci -m 1;""" % minilay)
        for index, minibuild in enumerate(pythonmbs):
            open('%s/minibuild-100%s' % (minilay, index), 'w').write(minibuild)

        # fake 3 pythons.
        for minibuild in ['python-2.4', 'python-2.5', 'python-2.6']:
            open('%s/%s' % (minilay, minibuild), 'w').write("""
[minibuild]
src_uri=http://hg.minitage.org/minitage
src_type=hg
install_method=buildout
category=dependencies""")

        open('%s/%s' % (minilay, 'meta-python'), 'w').write("""
[minibuild]
dependencies=python-2.4 python-2.5 python-2.6
src_uri=http://hg.minitage.org/minitage
src_type=hg
category=meta
install_method=buildout""")

        os.system("""
                  cd %s ;
                  echo "[paths]">.hg/hgrc
                  echo "default = %s">>.hg/hgrc
                  hg add;
                  hg ci -m 2;""" % (minilay, minilay))


    def tearDown(self):
        """."""
        os.unlink(self.minimerge10)
        os.chdir(__cwd__)
        shutil.rmtree(os.path.expanduser(path))

#    def testGetInstalledPackage(self):
#        """testGetInstalledPackage"""
#        package = test_common.createPackage(path)
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'foo']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        py24 = minimerge._find_minibuild('python-2.4')
#        py24p = minimerge.get_install_path(py24)
#        if os.path.exists(py24p): shutil.rmtree(py24p)
#        py24.src_uri = 'file://%s' % package
#        py24.src_type = 'static'
#        os.makedirs(py24p)
#        minimerge.record_minibuild(py24)
#        mb = minimerge.get_installed_minibuild(py24)
#        # packasdge is not installed properly
#        self.assertTrue(mb is None)
#        if os.path.exists(py24p): shutil.rmtree(py24p)
#        minimerge._fetch(py24)
#        minimerge._do_action('install', [py24])
#        # package is installed
#        mb = minimerge.get_installed_minibuild(py24)
#        self.assertEquals(os.path.join(py24p, '.minitage', 'minibuild'), mb.path)
#
#    def testRecordMinibuild(self):
#        """testRecordMinibuild"""
#        package = test_common.createPackage(path)
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'foo']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        py24 = minimerge._find_minibuild('python-2.4')
#        py24p = minimerge.get_install_path(py24)
#        if os.path.exists(py24p): shutil.rmtree(py24p)
#        py24.src_uri = 'file://%s' % package
#        py24.src_type = 'static'
#        os.makedirs(py24p)
#        minimerge.record_minibuild(py24)
#        self.assertTrue(
#            '[minibuild]' in open(
#                os.path.join(py24p, '.minitage', 'minibuild')
#            ).read()
#        )
#
#    def testIsPackageToBeInstalled(self):
#        """testIsPackageToBeInstalled"""
#        package = test_common.createPackage(path)
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'foo']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        py24 = minimerge._find_minibuild('python-2.4')
#        py24p = minimerge.get_install_path(py24)
#        py24.src_uri = 'file://%s' % package
#        py24.src_type = 'static'
#        if os.path.exists(py24p): shutil.rmtree(py24p)
#        minimerge._action = 'delete'
#        self.assertFalse(minimerge.is_package_to_be_installed(py24))
#        minimerge._action = 'install'
#        self.assertTrue(minimerge.is_package_to_be_installed(py24))
#        minimerge._fetch(py24)
#        self.assertTrue(minimerge.is_package_to_be_installed(py24))
#        minimerge._do_action('install', [py24])
#        self.assertFalse(minimerge.is_package_to_be_installed(py24))
#
#    def testIsPackageToBeReInstalled(self):
#        """testIsPackageToBeReInstalled"""
#        package = test_common.createPackage(path)
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'foo']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        py24 = minimerge._find_minibuild('python-2.4')
#        py24p = minimerge.get_install_path(py24)
#        py24.src_uri = 'file://%s' % package
#        py24.src_type = 'static'
#        if os.path.exists(py24p): shutil.rmtree(py24p)
#        minimerge.action = 'reinstall'
#        self.assertTrue(minimerge.is_package_to_be_installed(py24))
#        self.assertFalse(minimerge.is_package_to_be_reinstalled(py24))
#        # we cant reinstall as it is not installed
#        minimerge._fetch(py24)
#        self.assertTrue(minimerge.is_package_to_be_installed(py24))
#        self.assertFalse(minimerge.is_package_to_be_reinstalled(py24))
#        # we cant reinstall as it is not the wanted action
#        minimerge._do_action('install', [py24])
#        minimerge._action = 'install'
#        self.assertFalse(minimerge.is_package_to_be_reinstalled(py24))
#        self.assertFalse(minimerge.is_package_to_be_installed(py24))
#        # reinstalling
#        minimerge._action = 'reinstall'
#        self.assertTrue(minimerge.is_package_to_be_reinstalled(py24))
#        self.assertFalse(minimerge.is_package_to_be_installed(py24))
#        # reinstall will run always
#        minimerge._do_action('reinstall', [py24])
#        self.assertTrue(minimerge.is_package_to_be_reinstalled(py24))
#        self.assertFalse(minimerge.is_package_to_be_installed(py24))
#
#    def testIsPackageToBeUpdated(self):
#        """testIsPackageToBeUpdated"""
#        package = test_common.createPackage(path)
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'foo']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        py24 = minimerge._find_minibuild('python-2.4')
#        py24p = minimerge.get_install_path(py24)
#        py24.src_uri = 'file://%s' % package
#        py24.src_type = 'static'
#        if os.path.exists(py24p): shutil.rmtree(py24p)
#        minimerge.action = 'install'
#        minimerge._fetch(py24)
#        # package is not installed or just installed, no update
#        self.assertFalse(minimerge.is_package_to_be_updated(py24))
#        minimerge._do_action('install', [py24])
#        self.assertFalse(minimerge.is_package_to_be_updated(py24))
#        minimerge._action = 'install'
#        self.assertFalse(minimerge.is_package_to_be_updated(py24))
#        # we have enabled update unconditionnally
#        minimerge._update = True
#        self.assertTrue(minimerge.is_package_to_be_updated(py24))
#        minimerge._do_action('install', [py24])
#        self.assertTrue(minimerge.is_package_to_be_updated(py24))
#        # new revision
#        minimerge._action = 'install'
#        minimerge._update = False
#        content = open(py24.path).read()
#        self.assertFalse(minimerge.is_package_to_be_updated(py24))
#        fic = open(py24.path, 'w')
#        fic.write(content+'\nrevision=666\n')
#        fic.close()
#        py24 = api.Minibuild(path=py24.path, config=minimerge._config)
#        self.assertTrue(minimerge.is_package_to_be_updated(py24))
#        # new revision is installed, no update
#        minimerge._do_action('install', [py24])
#        self.assertFalse(minimerge.is_package_to_be_updated(py24))
#
#    def testIsPackageToBeDeleted(self):
#        """testIsPackageToBeDeleted"""
#        package = test_common.createPackage(path)
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'foo']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        py24 = minimerge._find_minibuild('python-2.4')
#        py24p = minimerge.get_install_path(py24)
#        py24.src_uri = 'file://%s' % package
#        py24.src_type = 'static'
#        if os.path.exists(py24p): shutil.rmtree(py24p)
#        minimerge.action = 'install'
#        # package is not install, no delete
#        self.assertFalse(minimerge.is_package_to_be_deleted(py24))
#        minimerge._fetch(py24)
#        self.assertFalse(minimerge.is_package_to_be_deleted(py24))
#        minimerge._do_action('install', [py24])
#        # action is not delete
#        self.assertFalse(minimerge.is_package_to_be_deleted(py24))
#        # package must be deleted
#        minimerge._action = 'delete'
#        self.assertTrue(minimerge.is_package_to_be_deleted(py24))
#        minimerge._do_action('delete', [py24])
#        # package is not installed no more, no delete
#        self.assertFalse(minimerge.is_package_to_be_deleted(py24))
#
#    def testIsPackageToBeFetched(self):
#        """testIsPackageToBeFetched"""
#        package = test_common.createPackage(path)
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'foo']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        py24 = minimerge._find_minibuild('python-2.4')
#        py24p = minimerge.get_install_path(py24)
#        if os.path.exists(py24p): shutil.rmtree(py24p)
#        py24.src_uri = 'file://%s' % package
#        py24.src_type = 'static'
#        self.assertTrue(minimerge.is_package_src_to_be_fetched(py24))
#        os.makedirs(py24p)
#        # package is not installed and unrelated
#        self.assertRaises(core.MinimergeError, minimerge.is_package_src_to_be_fetched, py24)
#        #fic = open(os.path.join(py24p, 'buildout.cfg'), 'w')
#        #fic.write()
#        #fic.close()
#        # package is incomplete
#        #self.assertTrue(minimerge.is_package_src_to_be_fetched(py24))
#        if os.path.exists(py24p): shutil.rmtree(py24p)
#        minimerge._fetch(py24)
#        minimerge._do_action('install', [py24])
#        # user package is fetched / up to date
#        self.assertFalse(minimerge.is_package_src_to_be_fetched(py24))
#        # user needs update
#        minimerge._update = True
#        self.assertFalse(minimerge.is_package_src_to_be_fetched(py24))
#        # package has new revision
#        minimerge._update = False
#        content = open(py24.path).read()
#        fic = open(py24.path, 'w')
#        fic.write(content+'\nrevision=666\n')
#        fic.close()
#        py24 = api.Minibuild(path=py24.path, config=minimerge._config)
#        self.assertFalse(minimerge.is_package_src_to_be_fetched(py24))
#
#    def testIsPackageToBeSrcUpdated(self):
#        """testIsPackageToBeSrcUpdated"""
#        package = test_common.createPackage(path)
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'foo']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        py24 = minimerge._find_minibuild('python-2.4')
#        py24p = minimerge.get_install_path(py24)
#        if os.path.exists(py24p): shutil.rmtree(py24p)
#        py24.src_uri = 'file://%s' % package
#        py24.src_type = 'static'
#        os.makedirs(py24p)
#        # package is not installed
#        self.assertFalse(minimerge.is_package_src_to_be_updated(py24))
#        if os.path.exists(py24p): shutil.rmtree(py24p)
#        minimerge._fetch(py24)
#        minimerge._do_action('install', [py24])
#        # user package is fetched / up to date
#        self.assertFalse(minimerge.is_package_src_to_be_updated(py24))
#        # user needs update
#        minimerge._update = True
#        self.assertTrue(minimerge.is_package_src_to_be_updated(py24))
#        # package has new revision
#        minimerge._update = False
#        content = open(py24.path).read()
#        fic = open(py24.path, 'w')
#        fic.write(content+'\nrevision=666\n')
#        fic.close()
#        py24 = api.Minibuild(path=py24.path, config=minimerge._config)
#        self.assertTrue(minimerge.is_package_src_to_be_updated(py24))
#
#
#    def testGetRevisionAndHasNewRevision(self):
#        package = test_common.createPackage(path)
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'foo']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        py24 = minimerge._find_minibuild('python-2.4')
#        py24p = minimerge.get_install_path(py24)
#        if os.path.exists(py24p): shutil.rmtree(py24p)
#        py24.src_uri = 'file://%s' % package
#        py24.src_type = 'static'
#        minimerge._fetch(py24)
#        minimerge._do_action('install', [py24])
#        # first install version 1
#        rev = minimerge.get_installed_revision(py24)
#        self.assertEquals(rev, 0)
#        self.assertFalse(minimerge.has_new_revision(py24))
#        content = open(py24.path).read()
#        fic = open(py24.path, 'w')
#        fic.write(content)
#        fic.write('\nrevision=1\n')
#        fic.close()
#        py24 = api.Minibuild(path=py24.path, config=minimerge._config)
#        self.assertTrue(minimerge.has_new_revision(py24))
#        py24.src_uri = 'file://%s' % package
#        py24.src_type = 'static'
#        minimerge._fetch(py24)
#        minimerge._do_action('install', [py24])
#        rev = minimerge.get_installed_revision(py24)
#        self.assertEquals(rev, 1)
#        py24 = api.Minibuild(path=py24.path, config=minimerge._config)
#        self.assertFalse(minimerge.has_new_revision(py24))
#
#    def testPackageMark(self):
#        """testPackageMark"""
#        package = test_common.createPackage(path)
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'foo']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        py24 = minimerge._find_minibuild('python-2.4')
#        py24p = minimerge.get_install_path(py24)
#        if os.path.exists(py24p): shutil.rmtree(py24p)
#        py24.src_uri = 'file://%s' % package
#        py24.src_type = 'static'
#        self.assertFalse(minimerge.get_package_mark(py24, 'fetched'))
#        minimerge.set_package_mark(py24, 'fetched', 'iamatest')
#        self.assertTrue(minimerge.is_package_marked(py24, 'fetched'))
#        self.assertTrue(
#            isinstance(
#                minimerge.get_package_mark_as_file(py24, 'fetched'),
#                file
#            )
#        )
#        self.assertEquals(minimerge.get_package_mark(py24, 'fetched'), 'iamatest')
#
#    def testFetchPackageMark(self):
#        """testFetchPackageMark"""
#        package = test_common.createPackage(path)
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'foo']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        py24 = minimerge._find_minibuild('python-2.4')
#        py24p = minimerge.get_install_path(py24)
#        if os.path.exists(py24p): shutil.rmtree(py24p)
#        py24.src_uri = 'file://%s' % package
#        py24.src_type = 'static'
#        self.assertFalse(minimerge.get_package_mark(py24, 'fetch'))
#        minimerge._fetch(py24)
#        self.assertTrue(minimerge.get_package_mark(py24, 'fetch'))
#
#    def testIsInstalled(self):
#       """testIsInstalled"""
#       package = test_common.createPackage(path)
#       sys.argv = [sys.argv[0], '--config',
#                   '%s/etc/minimerge.cfg' % path, 'foo']
#       opts = cli.do_read_options()
#       minimerge = api.Minimerge(opts)
#       py24 = minimerge._find_minibuild('python-2.4')
#       py24p = minimerge.get_install_path(py24)
#       if os.path.exists(py24p): shutil.rmtree(py24p)
#       mb = minimerge._find_minibuild('minibuild-0')
#       py24.src_uri = 'file://%s' % package
#       py24.src_type = 'static'
#       self.assertFalse(minimerge.is_installed(py24))
#       minimerge._fetch(py24)
#       self.assertFalse(minimerge.is_installed(py24))
#       minimerge._do_action('install', [py24])
#       self.assertTrue(minimerge.is_installed(py24))
#       # test reinstall
#       minimerge._do_action('install', [py24])
#       self.assertTrue(minimerge.is_installed(py24))
#
#    def testFindMinibuild(self):
#        """testFindMinibuild"""
#        # create minilays in the minilays dir, seeing if they get putted in
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'foo']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        mb = minimerge._find_minibuild('minibuild-0')
#        self.assertEquals('minibuild-0', mb.name)
#
#    def testComputeDepsWithNoDeps(self):
#        """testComputeDepsWithNoDeps
#        m0 depends on nothing"""
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'foo']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        computed_packages = minimerge._compute_dependencies(['minibuild-0'])
#        mb = computed_packages[0]
#        self.assertEquals('minibuild-0', mb.name)
#
#    def testSimpleDeps(self):
#        """testSimpleDeps
#        test m1 -> m0"""
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'foo']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        computed_packages = minimerge._compute_dependencies(['minibuild-1'])
#        mb = computed_packages[0]
#        self.assertEquals(mb.name, 'minibuild-0')
#        mb = computed_packages[1]
#        self.assertEquals(mb.name, 'minibuild-1')
#
#    def testChainedandTreeDeps(self):
#        """testChainedandTreeDeps
#        Will test that this tree is safe:
#              -       m3
#                      /
#                     m2
#                     / \
#                    m4 m1
#                     \/
#                     m0
#
#               -   m9
#                  / \
#                 m0 m3
#        """
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'foo']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        computed_packages = minimerge._compute_dependencies(['minibuild-3'])
#        wanted_list = ['minibuild-0', 'minibuild-4',
#                       'minibuild-1', 'minibuild-2', 'minibuild-3']
#        self.assertEquals([mb.name for mb in computed_packages], wanted_list)
#        computed_packages = minimerge._compute_dependencies(['minibuild-9'])
#        wanted_list = ['minibuild-0', 'minibuild-4', 'minibuild-1',
#                       'minibuild-2', 'minibuild-3', 'minibuild-9']
#        self.assertEquals([mb.name for mb in computed_packages], wanted_list)
#
#    def testRecursivity(self):
#        """testRecursivity
#        check that:
#             - m5  -> m6 -> m7
#             - m8  -> m8
#             - m10 -> m11 -> m12 -> m13 -> m10
#        will throw some recursity problems.
#        """
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'foo']
#        opts = cli.do_read_options()
#
#        minimerge = api.Minimerge(opts)
#        self.assertRaises(core.CircurlarDependencyError,
#                          minimerge._compute_dependencies, ['minibuild-6'])
#
#        minimerge = api.Minimerge(opts)
#        self.assertRaises(core.CircurlarDependencyError,
#                          minimerge._compute_dependencies, ['minibuild-8'])
#
#        minimerge = api.Minimerge(opts)
#        self.assertRaises(core.CircurlarDependencyError,
#                          minimerge._compute_dependencies, ['minibuild-13'])
#
#    def testMinibuildNotFound(self):
#        """testMinibuildNotFound
#        INOTINANYMINILAY does not exist"""
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'foo']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        self.assertRaises(core.MinibuildNotFoundError,
#                          minimerge._find_minibuild, 'INOTINANYMINILAY')
#
#    def testCutDeps(self):
#        """testCutDeps"""
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path,
#                    '--jump', 'minibuild-2',
#                    'minibuild-1', 'minibuild-2', 'minibuild-3']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        p =  minimerge._cut_jumped_packages(
#            minimerge._find_minibuilds(
#                ['minibuild-1', 'minibuild-2', 'minibuild-3']
#            )
#        )
#        self.assertEquals(
#            p,
#            minimerge._find_minibuilds(['minibuild-2', 'minibuild-3'])
#        )
#        minimerge._jump = '666'
#        q =  minimerge._cut_jumped_packages(
#            minimerge._find_minibuilds(
#                ['minibuild-1', 'minibuild-2', 'minibuild-3'])
#        )
#        self.assertEquals(
#            q,
#            minimerge._find_minibuilds(
#                ['minibuild-1', 'minibuild-2', 'minibuild-3']
#            )
#        )
#
#
#    def testFetchOffline(self):
#        """testFetchOffline"""
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, '--offline', 'minibuild-0']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        self.assertTrue(minimerge._offline)
#
#
#    def testSelectPython(self):
#        """testSelectPython.
#        Goal of this test is to prevent uneccesary python versions
#        to be built.
#        """
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'minibuild-1000']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        self.assertTrue(minimerge._action, 'install')
#
#        # we install a dependency which is not a egg
#        # result is false, no eggs dict in return.
#        computed_packages0, p0 = minimerge._select_pythons(
#            minimerge._compute_dependencies(['minibuild-1000']))
#        self.assertFalse(p0)
#
#        # we install a dep that require meta-python
#        # the dict must contains eggs 2.6
#        # available python = 2.4/2.5/2.6
#        self._packages = ['minibuild-1001']
#        computed_packages1, p1 = minimerge._select_pythons(
#            minimerge._compute_dependencies(['minibuild-1001']))
#        for i in ['2.6']:
#            self.assertTrue(i in p1['minibuild-1005'])
#            self.assertTrue('python-%s' % i
#                            in [c.name for c in computed_packages1]
#                           )
#        # we fake a python-2.4 installation
#        os.makedirs(
#            os.path.join(minimerge._prefix, 'dependencies', 'python-2.4')
#        )
#
#        # we install a dep that require meta-python
#        # the dict must contains eggs 2.4 as there is
#        # a python2.4 allready installed
#        self._packages = ['minibuild-1001']
#        computed_packages1, p1 = minimerge._select_pythons(
#            minimerge._compute_dependencies(['minibuild-1001']))
#        for i in ['2.4']:
#            self.assertTrue(i in p1['minibuild-1005'])
#            self.assertTrue('python-%s' % i
#                            in [c.name for c in computed_packages1]
#                           )
#
#        # we install a dep that require python-2.4 but depends
#        # on a python egg 'minibuild-1005'
#        # the dict must contains eggs part againts 2.4
#        # available python = 2.4
#        deps = minimerge._compute_dependencies(['minibuild-1002'])
#        self._packages = ['minibuild-1002']
#        computed_packages2, p2 = minimerge._select_pythons(deps)
#        self.assertTrue('python-2.4'
#                        in [c.name for c in computed_packages2]
#                       )
#        self.assertTrue('python-2.5'
#                        not in [c.name for c in computed_packages2]
#                       )
#        self.assertTrue('2.4' in p2['minibuild-1005'])
#        self.assertTrue('2.5' not in p2['minibuild-1005'])
#
#        # we install a dep that require python-2.5
#        # the dict must contains eggs ==  2.5
#        # available python = 2.5
#        deps = minimerge._compute_dependencies(['minibuild-1003'])
#        self._packages = ['minibuild-1003']
#        computed_packages3, p3 = minimerge._select_pythons(deps)
#        self.assertTrue('python-2.5'
#                        in [c.name for c in computed_packages3]
#                       )
#        self.assertTrue('python-2.4'
#                        not in [c.name for c in computed_packages3]
#                       )
#        self.assertTrue('2.5' in p3['minibuild-1005'])
#        self.assertTrue('2.4' not in p3['minibuild-1005'])
#
#        # we install a dep that require only a egg, no pytthon
#        # is specified specificly
#        # the dict must contains eggs ==  2.4
#        # available python = 2.4
#        # thus because we created a fake 'python-2.4'
#        self._packages = ['minibuild-1004']
#        computed_packages4, p4 = minimerge._select_pythons(
#            minimerge._compute_dependencies(['minibuild-1004']))
#        for i in ['2.4']:
#            self.assertTrue(i in p4['minibuild-1005'])
#            self.assertTrue('python-%s'%i
#                            in [c.name for c in computed_packages4]
#                           )
#        # we install an egg directly
#        # the dict must contains eggs ==  2.5/2.4/2.6
#        # available python (ones installed):  2.4
#        self._packages = ['minibuild-1005']
#        minimerge._packages = ['minibuild-1005']
#        computed_packagest, pt = minimerge._select_pythons(
#            minimerge._compute_dependencies(['minibuild-1005']))
#        for i in ['2.4']:
#            self.assertTrue(i in pt['minibuild-1005'])
#            self.assertTrue('python-%s'%i
#                            in [c.name for c in computed_packagest]
#                           )
#        for i in ['2.5', '2.6']:
#            self.assertFalse(i in pt['minibuild-1005'])
#            self.assertFalse('python-%s'%i
#                            in [c.name for c in computed_packagest]
#                           )
#
#
#    def testActionDelete(self):
#        """testActionDelete."""
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'minibuild-0']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#
#        ipath = '%s/dependencies/python-%s' % (path, '2.4')
#        test_common.make_dummy_buildoutdir(ipath)
#
#        py24 = minimerge._find_minibuild('python-2.4')
#        self.assertTrue(os.path.isdir(ipath))
#        minimerge._do_action('delete', [py24])
#        self.assertTrue(not os.path.isdir(ipath))
#
#    def testActionInstall(self):
#        """testActionInstall."""
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'minibuild-0']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#
#        for i in ['2.4', '2.5']:
#            test_common.make_dummy_buildoutdir(
#                '%s/dependencies/python-%s' % (path, i)
#            )
#
#        test_common.make_dummy_buildoutdir(
#            '%s/eggs/%s' % (path, 'minibuild-1005')
#        )
#
#        my_d = {'python-2.4': '2.4'}
#        py24 = minimerge._find_minibuild('python-2.4')
#        respy24 = '%s/dependencies/python-2.4/testres' % path
#        minimerge._do_action('install', [py24], my_d)
#        py25 = minimerge._find_minibuild('python-2.5')
#        respy25 = '%s/dependencies/python-2.5/testres' % path
#
#        minimerge._do_action('install', [py24], my_d)
#        self.assertEquals(open(respy24,'r').read(), 'bar')
#        minimerge._do_action('install', [py25], my_d)
#        self.assertEquals(open(respy25,'r').read(), 'bar')
#
#        m1005 = minimerge._find_minibuild('minibuild-1005')
#        m1005res =   '%s/eggs/minibuild-1005/testres' % path
#        m1005res24 = '%s/eggs/minibuild-1005/testres2.4' % path
#        m1005res25 = '%s/eggs/minibuild-1005/testres2.5' % path
#        minimerge._do_action('install', [m1005], {'minibuild-1005' :
#                                                  ['2.4', '2.5']})
#        self.assertEquals(open(m1005res24,'r').read(), '2.4')
#        self.assertEquals(open(m1005res25,'r').read(), '2.5')
#        self.assertFalse(os.path.isfile(m1005res))
#
#    def testInvalidAction(self):
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'minibuild-0']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        test_common.make_dummy_buildoutdir(
#            '%s/dependencies/python-2.4' % path
#        )
#        py = minimerge._find_minibuild('python-2.4')
#        self.assertRaises(core.ActionError,
#                          minimerge._do_action,
#                          'invalid',
#                          [py]
#                         )
#
#    def testSync(self):
#        """testSync."""
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'minibuild-0']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        os.system("cd %s;hg up -r 0" % minilay)
#        self.assertFalse(os.path.isfile('%s/%s' % (minilay, 'minibuild-1000')))
#        minimerge._sync()
#        self.assertTrue(
#            os.path.isdir(
#                '%s/minilays/%s' %
#                (path, 'dependencies')
#            )
#        )
#        self.assertTrue(os.path.isfile('%s/%s' % (minilay, 'minibuild-1000')))
#
#
#    def testFetchOnline(self):
#        """testFetchOnline"""
#        sys.argv = [sys.argv[0], '--config',
#                    '%s/etc/minimerge.cfg' % path, 'minibuild-0']
#        opts = cli.do_read_options()
#        minimerge = api.Minimerge(opts)
#        self.assertFalse(minimerge._offline)
#        minimerge._fetch(minimerge._find_minibuild('minibuild-0'))
#        self.assertTrue(os.path.isdir('%s/eggs/minibuild-0/.hg' % path))



def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(testMinimerge))
    return suite

if __name__ == '__main__':
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(testMinimerge))
    unittest.TextTestRunner(verbosity=2).run(suite)


